﻿using System;
using System.Collections.ObjectModel;
using System.Data.Services.Client;
using System.Windows.Browser;
using TreeViewLoadOnDemand.NorthwindServiceReference;

namespace TreeViewLoadOnDemand
{
	public class NorthwindDataSource
	{
		static NorthwindDataSource()
		{
			Uri appUri = HtmlPage.Document.DocumentUri;

			Service = new NorthwindEntities(new Uri(appUri.AbsoluteUri.Replace("Default.aspx", "NorthwindService.svc"), UriKind.Absolute));
		}

		public static NorthwindEntities Service { get; private set; }

		public NorthwindDataSource()
		{
			this.Categories = new ObservableCollection<Category>();

			Service.CategorySet.BeginExecute(
				(IAsyncResult result) => EntitiesLoaded<Category>(result, this.Categories),
				Service.CategorySet);
		}

		public ObservableCollection<Category> Categories { get; private set; }

		public static void BeginLoadingProducts(Category category)
		{
			DataServiceQuery<Product> categoryProducts = NorthwindDataSource.Service
				.CreateQuery<Product>(string.Format("CategorySet({0})/Products", category.CategoryID))
				.Expand("Supplier")
				.Expand("Category");

			categoryProducts.BeginExecute(
				(IAsyncResult result) => EntitiesLoaded<Product>(result, category.Products),
				categoryProducts);
		}

		private static void EntitiesLoaded<T>(IAsyncResult result, Collection<T> entities)
		{
			DataServiceQuery<T> query = result.AsyncState as DataServiceQuery<T>;
			foreach (T entity in query.EndExecute(result))
			{
				entities.Add(entity);
			}
		}
	}
}